#!/opt/nimbus/bin/perl

use strict;
use Cwd;
use Getopt::Std;


my $cwd;
my $perllib;

BEGIN
{
   $cwd = getcwd;

    my @dir = split(/\//, $cwd);
    my $dir;

    while($dir = pop(@dir))
    {
        last if ($dir eq 'probes');
    }
    push(@dir, 'perllib');
    $perllib = join('/', @dir);
}

use lib $perllib;

use Nimbus::API;
use Nimbus::Session;
use Nimbus::CFG;

my $qosNull  = -987654321;
my $prgname  = 'cue_exec';
my $version  = '1.5';
my $config;
my $user     = 'undefined';
my $password = 'undefined';
my $loglevel = 0;
my $logfile  = "$prgname.log";
my $next_run = 0;
my $interval = 300;
my $link;
my $apicount;
my $alarmUrl;
my $inactive_alarm;

my %qosDefinition;
my %ci;
my %monitor;
#my $met ="2.5.1:1";

sub nimLogAlarm
{
    my $log_level = shift;
    my $alarm_level = shift;
    my $message = shift;

    nimLog($log_level, $message);
    nimAlarm($alarm_level, $message);
}

###################################################
# DoWork - function called by dispatcher on timeout
#


sub doWork
{
 eval {
    my $now = time();
    return if ($now < $next_run);
    $next_run = $now + $interval;
    nimLog(1, "(doWork) - interval has passed");

    ######################
    # Enter your code here
    ######################

    if ($user eq 'undefined')
    {
        nimLogAlarm(0, NIML_MAJOR, "please define a valid CUE user in $config");
        exit(1);
    }
    if ($password eq 'undefined')
    {
        nimLogAlarm(0, NIML_MAJOR, "please define a valid CUE password in $config");
        exit(1);
    }
    
    nimLog(2, "cwd=$cwd");
    my $nkey = 'undefined';
    my $curl = "curl --silent --insecure --cookie \"$cwd/cookies\" --cookie-jar \"$cwd/cookies\"";
    my $api = 'https://api.cloudmonitor.ca.com/latest';
    open LOGIN, "$curl --data \"user=$user&password=$password\" $api/acct_login |";
	# Login to retrieve nkey from Cloudmonitor
    while(<LOGIN>)
    {
        my $line = $_;
        chomp($line);
        nimLog(3, "acct_login line=$line");
        if ($line =~ /<nkey>(\S+)<\/nkey>/)
        {
            $nkey = $1;
            nimLog(2, "nkey=$nkey");
			#Retrieve the current list of rules from the CloudMonitor instance 
            open RULE_GET, "$curl --data \"nkey=$nkey\" $api/rule_get |";
				$apicount += 1.0;
			    my $host;
				my $name;
				my $wmtype;
				my $rid;
				my $active;
				my $logurl="https://dashboard.cloudmonitor.nimsoft.com/en/logviewer.php?vrid=";
				
			#Process each rule to get the host infromation status, and verdict
            while (<RULE_GET>)
            {
                my $line = $_;
                chomp($line);
                nimLog(3, "rule_get line=$line");
			
				
				if ($line =~ /<name>(.*)<\/name>/)
                {
                    $name = $1;
					
                    nimLog(3, "Reading in Rule: $name");
					next;
				}
				
				if ($line =~ /<rid>(.*)<\/rid>/)
                {
                    $rid = $1;
                    nimLog(3, "Found Rule ID=$rid");
					next;
				}
				if ($line =~ /<active>(.*)<\/active/)
                {
                    $active = $1;
                    nimLog(3, "Set Active flag to $active");
					next;
				}
				if ($line =~ /<host>(.*)<\/host>/)
				{
					$link ="<a href=\"$1\">$name<\a>";
					
					# Remove www or www3 etc for the hostname.
					if ($1 =~ /^w{3}\d?\.(.*)/)
					{
						nimLog(3,"Removed triple dub on $line: $1 ");
						$host = $1;
					}
					else
					{
						
						$host = $1;
					}
					nimLog(3,"Set Host: $1 \n");
					# If the CI hasn't already been created, create a CI for this host.
					if(!$monitor{$name})
					{
						#TODO: Handle multiple IPs. Public IPs that change will have multiple CI entries. Possibly require a usertag in Cloudmonitor to handle this.
						if ($active eq "n" and $inactive_alarm eq "no")
						{
								nimLog(3,"Skipping CI creation for: $name ");
						}
						else 
						{
							nimLog(3,"Creating CI for: $name ");
							my $res = $ci{$name} = createCi($name, $host);
							nimLog(3, "CI Creation Result: $res ");
							nimLog(3, "Created CI for name=$name at host=$host With CI Reference: " . ref($ci{$name}) );
							
							$monitor{$name} = 1;
						}
					}
					#Send informational alarm for inactive monitors (Optional in config)
					if ($active eq "n" and $inactive_alarm eq "yes")
					{
						my ($qname,$qdesc,$met) = getQoSName('result');
						my $message ="$name configured, but is not currently monitored and has been set to inactive";
						my $level = 1;
						nimLog(2,"Sending Informational Inactive Alarm for $name ");
						if ($ci{$name}) { #Avoid using $ci($name) if the ci has not been created.
							ciAlarm($ci{$name}, $met, $level, $message,"","","","",$name);
						}
						
					}
					next;
				}
				
				if ($line =~ /<type>(.*)<\/type>/)
                {
					#Limit types of tests to process. Might also be helpful for USM Metrics, or clarification of host/target
                    $wmtype=$1;
					 nimLog(3, "Found type=$wmtype\n");
					next;
				}

				if ($line =~ /<\/rule>/)
				{
					my $verdict = 'down';
					my $alarm_reason;
					my $errcode = "";
					nimLog(2, "Processing Rule called $name");
					# Skip unsupported (Untested) monitor types. Some monitors don't have "host" set correctly etc.
					if ($wmtype !~ /https?|browser|script|ping|scp/)
					{
						nimLogAlarm(1,3,"$name monitor is of type: $wmtype which is not as yet supported for integration into Nimsoft Monitor.");
						$verdict = 'inactive';
					}
					elsif ($active eq "n") 
					{
						#$notactive = 0;
						nimLog(2, "Skipping $name.  This Monitor is NOT active. ");
						$verdict = 'inactive';
					}

					else
					{
						open RULE_DON, "$curl --data \"nkey=$nkey&acct=$user&name=$name&num=1&cache_only=y\" $api/rule_don |";
						$apicount += 0.1;
						while(<RULE_DON>)
						{
							my $line = $_;
							chomp($line);
							my @target = ("result", "ttime", "rtime", "ctime", "ptime",
										  "dtime", "dsize","descr"); 			#  Potential to organize by city using"cp_city","cp_country");
							my $target;
							my $unit;
							my $abrv;
							my $alarm=0;
							
							
							
							nimLog(3, "rule_don line=$line");
							
							if ($line =~ /<verdict>(\S+)<\/verdict>/)
							{
								$verdict = $1;
								nimLog(3, "verdict=$verdict");
							}
							if ($line =~ /<descr>(.*)<\/descr>/)
							{
								
								nimLog(3, "Alarm Flagged for: $name");
								$alarm_reason = $1;
								next;
							}
							#Process rule for QoS Metrics defined in @target 
							for $target(@target)
							{
								
								my $pattern = "<$target>(\\S+)</$target>";
								
								if ($line =~ /$pattern/)
								{
									nimLog(2, "Checking target = $target $pattern on line=$line");
																		
									my $value = $1;
									
									#Grab error code if result contains an error code
									if ($target eq 'result')
									{
										nimLog(2, "Checking result for: $name Result: $value");
										if($value > 0)
										{	
											$alarm = 1;
											$errcode = $value;
											nimLog(2,"Recieved errorcode: $value. Sending QOS $host: $target -> $name");
											publishQoS(uc($prgname), $name, $target, $value, $host);
											next;
										}
									}
									#skip if vaule is null, or if the monitor is inactive.
									if ($value ne '' and $verdict ne 'inactive')
									{
										nimLog(2,"Publishing QOS $host: $target -> $name");
										publishQoS(uc($prgname), $name, $target, $value, $host);
									}
								}
							}


							
							
							
							
						}
					
					}
					
					nimLog(3,"Finished processing $name.  Determining Alarm status on verdict = $verdict");
					#Send Alarms associated to CIs.
					my $level = 5;
					my ($qname,$qdesc,$met) = getQoSName('result');
					my $message ="$name reports $verdict. $alarm_reason - Error Code: $errcode.";
					if ($alarmUrl) {$message .= " View $name logs at " . $logurl . $rid ;}
					if ($verdict eq "down")
					{
						nimLog(1,"Sending Critical Alarm for $name Status: $verdict");
						if ($ci{$name}) { #Avoid using $ci($name) if the ci has not been created.
							ciAlarm($ci{$name}, $met, $level, $message,"","","","suppKey" . $name,$name);
						}
					}
					elsif ($verdict ne "inactive")
					{
						$level =0;
						nimLog(3,"Sending clear Alarm for $name Status: $verdict");
						if ($ci{$name}) { #Avoid using $ci($name) if the ci has not been created.
							ciAlarm($ci{$name}, $met, $level, $message,"","","","suppKey" . $name,$name);
						}
					}	
					
						
					close RULE_DON;
					
				}
				
				
            }
			
            close RULE_GET;
			nimLog(1,"Finished Processing. API Count used: $apicount  Logging out of Cloudmonitor API.");
            open LOGOUT, "$curl --data \"nkey=$nkey\" $api/acct_logout |";
            while (<LOGOUT>)    
            {
                $line = $_;
                nimLog(3, "acct_logout line=$line");
            }
            close LOGOUT;
        }
    }
    close LOGIN;
    if ($nkey eq 'undefined')
    {
        nimLogAlarm(0, NIML_MAJOR, "could not login to CUE, has curl been installed" .
            "and is the CU API user/password correct?");
    }
  };
     if ( $@ ) {
       nimLog( 0, "PROBE CRASHED: $@" );
     die;
  } 
}

sub createCi
{
	
	my $ciName = shift;
	my $ciHost = shift;
	my $ciType = "2.5";
	
	nimLog(3, "Creating CI for name = $ciName  " . 
				"host = $ciHost  " .
				"type = $ciType  ");
	my $ci = ciOpenRemoteDevice($ciType, $ciName, $ciHost);
	#my $rc =ciClose($ci);
	nimLog(3, "Created ci for $ciName");
	return $ci;
}

sub getUnit
{
	my $target = shift;
	my $un ='';
	my $ab='';
	   #my $met ="2.5.1:1";
	   
	if ($target =~ m/.time/)
	{
		$un = "Milliseconds";
		$ab = "ms";
	}
	elsif ($target eq 'result')
	{
		$un = "Status";
		$ab = "status";
	}
	elsif ($target =~ m/.size/)
	{
		$un = "KiloBytes";
		$ab = "KB";
	}
	else
	{
		$un = "Units";
		$ab = "unit";
	}
	return ($un, $ab);
}
sub publishQoS
{
	my $probe = shift;
    my $name = shift;
    my $target = shift;
    my $samplevalue = shift;
	my $host = shift;
	my ($unit, $abrv) = getUnit($target);
    my $samplerate = $interval;
	my ($qosName, $qosDesc) = getQoSName($target);  
	my $qosGroup = "QOS_APPLICATION";
	
	
	
	nimLog(2, "Monitor name: $name");
    nimLog(2, "(publishQoS) - QoSname=$qosName" .
              " QoSDescription=$qosDesc" .
              " target=$target" .
              " samplevalue=$samplevalue" .
              " samplerate=$samplerate" . 
			  " host=$host" .
			  " unit=$unit" .
			  " abreviated=$abrv");
    if (!$qosDefinition{$target})
    {
        #We only want to send the definitions ONCE!
        nimLog(2, "nimQoSSendDefinition name=$name");
        nimQoSSendDefinition ( $qosName, $qosGroup, $qosDesc, $unit,$abrv);
        $qosDefinition{$target} = 1;

    }
	
    if (my $qos = nimQoSCreate($qosName, $host, $samplerate, -1))
    {
		my ($qname,$qdesc,$met) = getQoSName($target);
		nimLog(1,"Calling ciBindQos with these parameters: HANDLE: $ci{$name} .QOS: $qos .Metric: $met");
		if ($ci{$name}) { #Avoid using $ci($name) if the ci has not been created.
			ciBindQoS($ci{$name}, $qos, $met);
		}
        if ($samplevalue == $qosNull)
        {
            nimQoSSendNull ($qos, $target);
        }
        else
        {
			
            nimQoSSendValue($qos, $name, $samplevalue);
        }
        nimQoSFree($qos);
		#ciUnBindQoS($qos); ####THIS DOES NOT SEEM TO WORK. errors at runtime.
		
    }
	else 
	{
		nimLog(1,"UNABLE TO SEND QOS for $name.  qos: $qos");
	}
}
sub getQoSName
{
	my $target = shift;
	my $qosName;
	my $qosDesc;
	my $ciMet;
	#"result", "ttime", "rtime", "ctime", "ptime",
    # "dtime", "dsize'
	nimLog(2,"Getting QoSName for: $target");
	if ($target eq 'result' ) 
	{
		$qosName = "QOS_CUE_MONITOR_STATUS";
		$qosDesc = "Error Code reported(No Error = 0).";
		$ciMet   = "2.2.2.2:2";
	}
	elsif ($target eq 'ttime' ) 
	{
		$qosName = "QOS_CUE_TOTAL_TIME";
		$qosDesc = "Total response time of the monitor measured in milliseconds.";
		$ciMet   = "2.2.2.2:1";
	}
	elsif ($target eq 'rtime' ) 
	{
		$qosName = "QOS_CUE_RESOLVE_TIME";
		$qosDesc = " Resolution time (DNS) of the monitor measured in milliseconds.";
		$ciMet   = "2.2.2.2:4";
	}
	elsif ($target eq 'ctime') 
	{
		$qosName = "QOS_CUE_CONNECT_TIME";
		$qosDesc = "Connect time of the monitor measured in milliseconds.";
		$ciMet   = "2.2.2.2:4";
	}	
	elsif ($target eq 'ptime' ) 
	{
		$qosName = "QOS_CUE_PROCESSING_TIME";
		$qosDesc = "Processing time of the monitor measured in milliseconds.";
		$ciMet   = "2.2.2.2:9";
	}
	elsif ($target eq 'dtime' ) 
	{
		$qosName = "QOS_CUE_DOWNLOAD_TIME";
		$qosDesc = "Download time of the monitor measured in milliseconds.";
		$ciMet   = "2.2.2.2:5";
	}
	elsif ($target eq 'dsize' ) 
	{
		$qosName = "QOS_CUE_DOWNLOAD_SIZE";
		$qosDesc = "Size of the downloaded target monitor measured in kiloBytes.";
		$ciMet   = "2.2.2.2:5";
	}	
	else {
		nimLog(0, "Bad target: $target. COULD NOT CREATE QOS_NAME: $qosName");
		exit;
	}
	#nimLog(2, "Created QOSNAME: $qosName and Description: $qosDesc");
	return ($qosName, $qosDesc, $ciMet);
}


###################
# Service functions
#
sub restart
{
    nimLog(1, "(restart) - got restarted");
}

sub timeout {
    nimLog(2, "(timeout) - got kicked ");
    doWork();
}

#############################
# Signal handler - Ctrl-Break
#
sub ctrlc
{
    exit;
}

############
# MAIN ENTRY
#
my %options;
getopts("d:l:i:",\%options);

$SIG{INT} = \&ctrlc;
 
$config = Nimbus::CFG->new("$prgname.cfg");
$user = $config->{setup}->{user} || "undefined";
$password = $config->{setup}->{password} || "undefined";
$loglevel = $options{d} || $config->{setup}->{loglevel}|| 0;
$logfile = $options{l} || $config->{setup}->{logfile} || "$prgname.log";
$interval= $options{i} || $config->{setup}->{interval}|| 300;
$alarmUrl = $options{u} || $config->{setup}->{alarmurl}||1;
$inactive_alarm = $options{n} || $config->{setup}->{inactive_alarm}||"no";

nimLogSet($logfile, $prgname, $loglevel, 1);

nimLog(0,"----------------- Starting  (pid: $$) ------------------");

my $sess;
$sess = Nimbus::Session->new("$prgname");
$sess->setInfo($version,"Nimsoft Corporation");

if ($sess->server (NIMPORT_ANY,\&timeout,\&restart)==0)
{
    nimLog(1, "server session is created");
}
else
{
    nimLog(0, "unable to create server session");
    exit(1);
}

$sess->dispatch();
nimLog(0, "Received STOP, terminating program");
nimLog(0, "Exiting program");
exit;

